/* prefs.c */



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gconf/gconf-client.h>

#include "support.h"
#include "puzzle.h"
#include "interface.h"
#include "prefs.h"
#include "gfx.h"


#define DEFAULT_SIZE     4
#define DEFAULT_OVERLAY  OVERLAY_WHITE
#define DEFAULT_RANDOM   FALSE
#define DEFAULT_KEY_UP   GDK_Up
#define DEFAULT_KEY_DN   GDK_Down
#define DEFAULT_KEY_LT   GDK_Left
#define DEFAULT_KEY_RT   GDK_Right

extern Puzzle *puzzle;
extern GtkWidget *app;
extern gint image_px;

static GtkWidget *dialog_prefs = NULL;
static GtkWidget *button_move[4];
static GtkWidget *entry_move[4];
static GtkWidget *optionmenu_size = NULL;
static GtkWidget *optionmenu_image = NULL;
static GtkWidget *menu_image;
static GtkWidget *menu_item_custom;

GConfClient *skoosh_gconf_client = NULL;
gchar       *fname_img_custom = NULL;
KeyID       key[4];
Overlay     overlay;
gint        img_id;



ImgFile img_file[] = {
        {
          NULL,
          N_("Custom"),
          0 /* (0 = allow any size) */
        },
        {
          PACKAGE_PIXMAP_DIR G_DIR_SEPARATOR_S "default.jpg",
          N_("Numbers"),
          4 /* (4 = restrict this theme to 4x4 tiles) */
        },
        {
          PACKAGE_PIXMAP_DIR G_DIR_SEPARATOR_S "griffs-summer-breeze.jpg",
          N_("Summer Breeze"),
          0
        }
        /* (add new images here) */
};
gint n_img_files = sizeof(img_file) / sizeof(img_file[0]);



static gint 
skoosh_gconf_get_int (gchar *key, gint default_int)
{
        GConfValue *value = NULL, *schema_value;
        gint retval = default_int;

        value = gconf_client_get (skoosh_gconf_client, key, NULL);
        if (value != NULL && value->type == GCONF_VALUE_INT) {
                retval = gconf_value_get_int (value);
                gconf_value_free (value);
        }
        else {
                schema_value = gconf_client_get_default_from_schema
                        (skoosh_gconf_client, key, NULL);
                if (schema_value != NULL) {
                        retval = gconf_value_get_int (schema_value);
                        gconf_value_free (schema_value);
                }
        }
        return retval;
}



static gboolean
skoosh_gconf_get_bool (gchar *key, gboolean default_bool)
{
        GConfValue *value = NULL, *schema_value;
        gboolean retval = default_bool;

        value = gconf_client_get (skoosh_gconf_client, key, NULL);
        if (value != NULL && value->type == GCONF_VALUE_BOOL) {
                retval = gconf_value_get_bool (value); 
                gconf_value_free (value);
        }
        else {
                schema_value = gconf_client_get_default_from_schema
                        (skoosh_gconf_client, key, NULL);
                if (schema_value != NULL) {
                        retval = gconf_value_get_bool (schema_value);
                        gconf_value_free (schema_value);
                }
        }
        return retval;
}



static gchar*
skoosh_gconf_get_string (gchar *key)
{
        GConfValue *value = NULL, *schema_value;
        gchar *retval = NULL;

        value = gconf_client_get (skoosh_gconf_client, key, NULL);
        if (value != NULL && value->type == GCONF_VALUE_STRING) {
                retval = g_strdup (gconf_value_get_string (value));
                gconf_value_free (value);
        }
        else {
                schema_value = gconf_client_get_default_from_schema
                        (skoosh_gconf_client, key, NULL);
                if (schema_value != NULL) {
                        retval = g_strdup (gconf_value_get_string (schema_value));
                        gconf_value_free (schema_value);
                }
        }
        return retval;
}



static void
prefs_load (void)
{
        gint size;

        fname_img_custom = skoosh_gconf_get_string ("/apps/skoosh/image");
        size = skoosh_gconf_get_int ("/apps/skoosh/size", DEFAULT_SIZE);
        overlay = skoosh_gconf_get_int ("/apps/skoosh/overlay", DEFAULT_OVERLAY);
        img_id = skoosh_gconf_get_int ("/apps/skoosh/img_id", IMG_ID_DEFAULT);
        puzzle_set_random (puzzle, skoosh_gconf_get_bool ("/apps/skoosh/random", DEFAULT_RANDOM));
        key[KEY_UP] = skoosh_gconf_get_int ("/apps/skoosh/keyup", DEFAULT_KEY_UP);
        key[KEY_DN] = skoosh_gconf_get_int ("/apps/skoosh/keydn", DEFAULT_KEY_DN);
        key[KEY_LT] = skoosh_gconf_get_int ("/apps/skoosh/keylt", DEFAULT_KEY_LT);
        key[KEY_RT] = skoosh_gconf_get_int ("/apps/skoosh/keyrt", DEFAULT_KEY_RT);

        /* sanity checks */
        if (size < 3 || size > 5) {
                size = DEFAULT_SIZE;
                gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/size", size, NULL);
        }
        if (overlay != OVERLAY_BLACK
            && overlay != OVERLAY_WHITE
            && overlay != OVERLAY_NONE) {
                overlay = DEFAULT_OVERLAY;
                gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/overlay", overlay, NULL);
        }
        if ((img_id < 0 || img_id >= n_img_files)
            || (img_id == IMG_ID_CUSTOM && fname_img_custom == NULL)) {
                img_id = IMG_ID_DEFAULT;
                gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/img_id", img_id, NULL);
        }
        if (fname_img_custom != NULL && strcmp (fname_img_custom, "none") == 0) {
                g_free (fname_img_custom);
                fname_img_custom = NULL;
        }

        puzzle_set_size (puzzle, size);
}


#if 0
static void
on_gconf_image_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
}
static void
on_gconf_size_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
}
static void
on_gconf_overlay_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
}
static void
on_gconf_image_id_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
}
static void
on_gconf_random_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
}
static void
on_gconf_keyup_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
}
static void
on_gconf_keydn_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
}
static void
on_gconf_keylt_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
}
static void
on_gconf_keyrt_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
}
#endif


void
prefs_init (gint argc, gchar **argv)
{
        gconf_init (argc, argv, NULL);
        skoosh_gconf_client = gconf_client_get_default ();
        gconf_client_add_dir (skoosh_gconf_client, "/apps/skoosh",
                              GCONF_CLIENT_PRELOAD_NONE, NULL);

        prefs_load ();

#if 0
        gconf_client_notify_add (skoosh_gconf_client, "/apps/skoosh/image",
                                 on_gconf_image_changed, NULL, NULL, NULL);
        gconf_client_notify_add (skoosh_gconf_client, "/apps/skoosh/size",
                                 on_gconf_size_changed, NULL, NULL, NULL);
        gconf_client_notify_add (skoosh_gconf_client, "/apps/skoosh/overlay",
                                 on_gconf_overlay_changed, NULL, NULL, NULL);
        gconf_client_notify_add (skoosh_gconf_client, "/apps/skoosh/img_id",
                                 on_gconf_image_id_changed, NULL, NULL, NULL);
        gconf_client_notify_add (skoosh_gconf_client, "/apps/skoosh/random",
                                 on_gconf_random_changed, NULL, NULL, NULL);
        gconf_client_notify_add (skoosh_gconf_client, "/apps/skoosh/keyup",
                                 on_gconf_keyup_changed, NULL, NULL, NULL);
        gconf_client_notify_add (skoosh_gconf_client, "/apps/skoosh/keydn",
                                 on_gconf_keydn_changed, NULL, NULL, NULL);
        gconf_client_notify_add (skoosh_gconf_client, "/apps/skoosh/keylt",
                                 on_gconf_keylt_changed, NULL, NULL, NULL);
        gconf_client_notify_add (skoosh_gconf_client, "/apps/skoosh/keyrt",
                                 on_gconf_keyrt_changed, NULL, NULL, NULL);
#endif
}


  
void
prefs_free (void)
{
        g_object_unref (G_OBJECT(skoosh_gconf_client));
        g_free (fname_img_custom);
}



static void
size_menu_set_sensitive (gboolean sensitive)
{
        if (optionmenu_size == NULL) return;
        gtk_widget_set_sensitive (optionmenu_size, sensitive);
}



void
prefs_set_image (gchar *fname)
{
        /* replace fname_img_custom and save to prefs */
        g_free (fname_img_custom);
        fname_img_custom = fname;
        gconf_client_set_string (skoosh_gconf_client, "/apps/skoosh/image",
                                 fname_img_custom, NULL);
}



void
prefs_set_image_id (gint id)
{
        /* replace img_id and update preferences */
        if (id != img_id) {
                img_id = id;
                gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/img_id", img_id, NULL);
                if (img_file[img_id].size != 0) {
                        gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/size", img_file[img_id].size, NULL);
                }
        }

        /* update what the prefs dialog's showing */
        if (dialog_prefs != NULL) {
                gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu_image), img_id);
                if (id == IMG_ID_CUSTOM) {
                        gtk_widget_set_sensitive (menu_item_custom, TRUE);
                }
                size_menu_set_sensitive (img_file[id].size == 0);
                gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu_size),
                                             puzzle_get_size (puzzle) - 3);
        }
}



static void
on_filesel_select (GtkWidget *w, GtkFileSelection *filesel)
{
        gchar *fname;

        fname = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION(filesel)));
        gtk_widget_destroy (GTK_WIDGET(filesel));
        if (fname != NULL && fname[0] != '\0') {
                if (gfx_load (fname, FALSE)) {
                        prefs_set_image (fname);
                        prefs_set_image_id (IMG_ID_CUSTOM);
                        return;
                }
                else {
                        file_error_dialog (fname, GTK_MESSAGE_WARNING);
                        prefs_set_image_id (img_id); /* restore the original setting */
                }
        }
        g_free (fname);
}



static void
on_filesel_cancel (GtkWidget *w, GtkFileSelection *filesel)
{
        prefs_set_image_id (img_id); /* restore the original setting */
        gtk_widget_destroy (GTK_WIDGET(filesel));
}



static GtkWidget*
create_dialog_filesel (void)
{
        GtkWidget *dialog_filesel;
        GtkWidget *ok_button;
        GtkWidget *cancel_button;


        dialog_filesel = gtk_file_selection_new (_("Please select an image"));
        gtk_container_set_border_width (GTK_CONTAINER (dialog_filesel), 10);
        gtk_window_set_modal (GTK_WINDOW(dialog_filesel), TRUE);
        gtk_window_set_transient_for (GTK_WINDOW(dialog_filesel), GTK_WINDOW(dialog_prefs));

        ok_button = GTK_FILE_SELECTION (dialog_filesel)->ok_button;
        gtk_widget_show (ok_button);
        GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);

        cancel_button = GTK_FILE_SELECTION (dialog_filesel)->cancel_button;
        gtk_widget_show (cancel_button);
        GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);

        g_signal_connect (G_OBJECT(GTK_FILE_SELECTION(dialog_filesel)->ok_button),
                          "clicked", (GtkSignalFunc)on_filesel_select, dialog_filesel);

        g_signal_connect (G_OBJECT(GTK_FILE_SELECTION(dialog_filesel)->cancel_button),
                          "clicked", (GtkSignalFunc)on_filesel_cancel, dialog_filesel);

        if (fname_img_custom != NULL) {
                gtk_file_selection_set_filename (GTK_FILE_SELECTION(dialog_filesel),
                                                 fname_img_custom);
        }

        return dialog_filesel;
}



static void
key_entries_set_sensitive (gboolean sensitive)
{
        gint i;

        for (i = 0; i < 4; i++) {
                gtk_widget_set_sensitive (entry_move[i], sensitive);
        }
}



static void
on_overlay_selected (GtkMenuItem *menuitem, gpointer new_overlay)
{
        if ((gint)new_overlay != overlay) {
                overlay = (gint)new_overlay;
                gfx_create_tileset ();
                gfx_redraw_drawarea (TRUE);
                gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/overlay",
                                      overlay, NULL);
        }
}



static void
on_size_selected (GtkMenuItem *menuitem, gpointer new_size)
{
        if ((gint)new_size == puzzle_get_size (puzzle)) return;
        if ((gint)new_size < 3 || (gint)new_size > 5) return;

        set_status (STATUS_CLEAR, NULL);

        puzzle_set_size (puzzle, (gint)new_size);
        gfx_create_tileset ();
        puzzle_display_tiles (puzzle);

        gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/size",
                              puzzle_get_size (puzzle), NULL);
}



static void
on_image_selected (GtkMenuItem *menuitem, gpointer new_img_id)
{
        gint id = (gint)new_img_id;
        
        if (id == IMG_ID_CUSTOM) {
                if (fname_img_custom != NULL) {
                        if (!gfx_load_img_id (IMG_ID_CUSTOM)) {
                                file_error_dialog (fname_img_custom, GTK_MESSAGE_WARNING);
                                /* forget this image */
                                gtk_widget_set_sensitive (menu_item_custom, FALSE);
                                prefs_set_image_id (img_id);
                                gconf_client_unset (skoosh_gconf_client, "/apps/skoosh/image", NULL);
                        }
                }
                else {
                        prefs_set_image_id (IMG_ID_CUSTOM);
                }
        }
        else if (id != img_id) {

                if (!gfx_load_img_id (id)) {
                        file_error_dialog (img_file[id].fname,
                                           GTK_MESSAGE_WARNING);
                }
        }
}



static void
on_button_browse_clicked (GtkButton *button, gpointer user_data)
{
        GtkWidget *filesel;
        filesel = create_dialog_filesel ();
        gtk_widget_show (filesel);
}


#if 0
static void
on_checkbutton_random_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
        puzzle_set_random (puzzle, !GTK_TOGGLE_BUTTON(togglebutton)->active);
        gconf_client_set_bool (skoosh_gconf_client, "/apps/skoosh/random", IS_RANDOM(puzzle), NULL);
}
#endif


static void
on_button_move_clicked (GtkWidget *button, gpointer key_select)
{
        key_entries_set_sensitive (FALSE);
        gtk_widget_set_sensitive (entry_move[(gint)key_select], TRUE);
        gtk_widget_grab_focus (entry_move[(gint)key_select]);
}



static void
on_key_selected (GtkWidget *entry, GdkEventKey *data)
{
        gint key_select;

        gtk_entry_set_text (GTK_ENTRY(entry), gdk_keyval_name (data->keyval));

        if (entry == entry_move[KEY_UP]) {
                key_select = KEY_UP;
                gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/keyup", data->keyval, NULL);
        }
        else if (entry == entry_move[KEY_DN]) {
                key_select = KEY_DN;
                gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/keydn", data->keyval, NULL);
        }
        else if (entry == entry_move[KEY_LT]) {
                key_select = KEY_LT;
                gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/keylt", data->keyval, NULL);
        }
        else {
                key_select = KEY_RT;
                gconf_client_set_int (skoosh_gconf_client, "/apps/skoosh/keyrt", data->keyval, NULL);
        }

        key[key_select] = data->keyval;
        gtk_widget_set_sensitive (entry, FALSE);
        gtk_widget_grab_focus (button_move[key_select]);
}



static GnomeUIInfo menu_decor_uiinfo[] =
{
        {
                GNOME_APP_UI_ITEM, N_("Black"),
                NULL,
                (gpointer) on_overlay_selected, (gpointer)OVERLAY_BLACK, NULL,
                GNOME_APP_PIXMAP_NONE, NULL,
                0, (GdkModifierType) 0, NULL
        },
        {
                GNOME_APP_UI_ITEM, N_("White"),
                NULL,
                (gpointer) on_overlay_selected, (gpointer)OVERLAY_WHITE, NULL,
                GNOME_APP_PIXMAP_NONE, NULL,
                0, (GdkModifierType) 0, NULL
        },
        {
                GNOME_APP_UI_ITEM, N_("None"),
                NULL,
                (gpointer) on_overlay_selected, (gpointer)OVERLAY_NONE, NULL,
                GNOME_APP_PIXMAP_NONE, NULL,
                0, (GdkModifierType) 0, NULL
        },
        GNOMEUIINFO_END
};



static GnomeUIInfo menu_size_uiinfo[] =
{
        {
                GNOME_APP_UI_ITEM, N_("Three by Three"),
                NULL,
                (gpointer) on_size_selected, (gpointer)3, NULL,
                GNOME_APP_PIXMAP_NONE, NULL,
                0, (GdkModifierType) 0, NULL
        },
        {
                GNOME_APP_UI_ITEM, N_("Four by Four"),
                NULL,
                (gpointer) on_size_selected, (gpointer)4, NULL,
                GNOME_APP_PIXMAP_NONE, NULL,
                0, (GdkModifierType) 0, NULL
        },
        {
                GNOME_APP_UI_ITEM, N_("Five by Five"),
                NULL,
                (gpointer) on_size_selected, (gpointer)5, NULL,
                GNOME_APP_PIXMAP_NONE, NULL,
                0, (GdkModifierType) 0, NULL
        },
        GNOMEUIINFO_END
};

    
    
static const gchar*
theme_title (gint id)
{
        /* this lets gettext translate the theme menu */
        return img_file[id].title;
}



static void
fill_image_menu (void)
{
        GtkWidget *item;
        gint i = 0;

        for (i = 0; i < n_img_files; i++) {
                item = gtk_menu_item_new_with_label ( _(theme_title (i)));
                if (i == IMG_ID_CUSTOM) menu_item_custom = item;
                gtk_widget_show (item);
                gtk_menu_shell_append (GTK_MENU_SHELL(menu_image), item);
                g_signal_connect (GTK_OBJECT(item), "activate",
                        GTK_SIGNAL_FUNC(on_image_selected), (gpointer)i);
        }
        gtk_widget_set_sensitive (menu_item_custom, fname_img_custom != NULL);
}



static void
on_prefs_response (GtkWidget *w, int response_id, gpointer data)
{
        switch (response_id) {
        case GTK_RESPONSE_ACCEPT :
                gtk_widget_hide (dialog_prefs);
                break;
                /*
        case GTK_RESPONSE_HELP :
                break;
                */
        }
        return;
}



void
create_prefs_dialog (void)
{
        GtkWidget *notebook;
        GtkWidget *vbox_appearance;
        GtkWidget *table_appearance;
        GtkWidget *label_size;
        GtkWidget *label_decor;
        GtkWidget *label_image;
        GtkWidget *optionmenu_decor;
        GtkWidget *menu_decor;
        GtkWidget *menu_size;
        GtkWidget *button_browse;
        /* GtkWidget *checkbutton_random; */
        GtkWidget *label_appearance;
        GtkWidget *table_keyboard;
        GtkWidget *label_keyboard;
        GtkTooltips *tooltips;
        gint i;


        if (dialog_prefs != NULL) {
                key_entries_set_sensitive (FALSE);
                gtk_window_present (GTK_WINDOW(dialog_prefs));
                return;
        }

        tooltips = gtk_tooltips_new ();

        dialog_prefs = gtk_dialog_new_with_buttons (_("Skoosh Preferences"),
                                                    GTK_WINDOW (app),
                                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                                    /* GTK_STOCK_HELP, GTK_RESPONSE_HELP, */
                                                    GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
                                                    NULL);

        g_signal_connect (GTK_OBJECT(dialog_prefs), "destroy",
                          GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog_prefs);

        g_signal_connect (dialog_prefs, "response",
                          G_CALLBACK(on_prefs_response), &dialog_prefs);

        gtk_window_set_resizable (GTK_WINDOW(dialog_prefs), FALSE);

        notebook = gtk_notebook_new ();
        gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog_prefs)->vbox), notebook, TRUE, TRUE, 0);
        gtk_widget_show (notebook);

        vbox_appearance = gtk_vbox_new (FALSE, 0);
        gtk_widget_show (vbox_appearance);
        gtk_container_add (GTK_CONTAINER (notebook), vbox_appearance);

        table_appearance = gtk_table_new (4, 2, FALSE);
        gtk_widget_show (table_appearance);
        gtk_box_pack_start (GTK_BOX (vbox_appearance), table_appearance, TRUE, TRUE, 0);
        gtk_container_set_border_width (GTK_CONTAINER (table_appearance), 12);
        gtk_table_set_row_spacings (GTK_TABLE (table_appearance), 6);
        gtk_table_set_col_spacings (GTK_TABLE (table_appearance), 12);

        label_size = gtk_label_new_with_mnemonic (_("_Puzzle Size:"));
        gtk_widget_show (label_size);
        gtk_table_attach (GTK_TABLE (table_appearance), label_size, 0, 1, 0, 1,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);
        gtk_label_set_justify (GTK_LABEL (label_size), GTK_JUSTIFY_LEFT);
        gtk_misc_set_alignment (GTK_MISC (label_size), 0, 0.5);

        label_decor = gtk_label_new_with_mnemonic (_("_Tile Borders:"));
        gtk_widget_show (label_decor);
        gtk_table_attach (GTK_TABLE (table_appearance), label_decor, 0, 1, 1, 2,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);
        gtk_label_set_justify (GTK_LABEL (label_decor), GTK_JUSTIFY_LEFT);
        gtk_misc_set_alignment (GTK_MISC (label_decor), 0, 0.5);

        optionmenu_decor = gtk_option_menu_new ();
        gtk_widget_show (optionmenu_decor);
        gtk_table_attach (GTK_TABLE (table_appearance), optionmenu_decor, 1, 2, 1, 2,
                          (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                          (GtkAttachOptions) (GTK_EXPAND), 0, 0);

        menu_decor = gtk_menu_new ();
        gnome_app_fill_menu (GTK_MENU_SHELL (menu_decor), menu_decor_uiinfo,
                             NULL, FALSE, 0);

        gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu_decor), menu_decor);

        optionmenu_size = gtk_option_menu_new ();
        gtk_widget_show (optionmenu_size);
        gtk_table_attach (GTK_TABLE (table_appearance), optionmenu_size, 1, 2, 0, 1,
                          (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                          (GtkAttachOptions) (GTK_EXPAND), 0, 0);

        menu_size = gtk_menu_new ();
        gnome_app_fill_menu (GTK_MENU_SHELL (menu_size), menu_size_uiinfo,
                             NULL, FALSE, 0);

        gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu_size), menu_size);


        optionmenu_image = gtk_option_menu_new ();
        gtk_widget_show (optionmenu_image);
        gtk_table_attach (GTK_TABLE (table_appearance), optionmenu_image, 1, 2, 2, 3,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);

        menu_image = gtk_menu_new ();
        gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu_image), menu_image);

        label_image = gtk_label_new_with_mnemonic (_("_Image:"));
        gtk_widget_show (label_image);
        gtk_table_attach (GTK_TABLE (table_appearance), label_image, 0, 1, 2, 3,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);
        gtk_label_set_justify (GTK_LABEL (label_image), GTK_JUSTIFY_LEFT);
        gtk_misc_set_alignment (GTK_MISC (label_image), 0, 0.5);


        button_browse = gtk_button_new_with_mnemonic (_("_Browse..."));
        gtk_widget_show (button_browse);
        gtk_table_attach (GTK_TABLE (table_appearance), button_browse, 1, 2, 3, 4,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);
        gtk_tooltips_set_tip (tooltips, button_browse, _("Alternatively, images can be dragged and dropped into the main window"), NULL);
#if 0
        checkbutton_random = gtk_check_button_new_with_mnemonic (_("_Missing tile is always bottom-right"));
        gtk_widget_show (checkbutton_random);
        gtk_box_pack_start (GTK_BOX (vbox_appearance), checkbutton_random, FALSE, FALSE, 0);
        gtk_container_set_border_width (GTK_CONTAINER (checkbutton_random), 12);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(checkbutton_random), !IS_RANDOM(puzzle));
#endif
        label_appearance = gtk_label_new_with_mnemonic (_("_Appearance"));
        gtk_widget_show (label_appearance);
        gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 0), label_appearance);
        gtk_label_set_justify (GTK_LABEL (label_appearance), GTK_JUSTIFY_LEFT);

        gtk_label_set_mnemonic_widget (GTK_LABEL(label_size), optionmenu_size);
        gtk_label_set_mnemonic_widget (GTK_LABEL(label_decor), optionmenu_decor);
        gtk_label_set_mnemonic_widget (GTK_LABEL(label_image), optionmenu_image);

        table_keyboard = gtk_table_new (4, 2, FALSE);
        gtk_widget_show (table_keyboard);
        gtk_container_add (GTK_CONTAINER (notebook), table_keyboard);
        gtk_container_set_border_width (GTK_CONTAINER (table_keyboard), 12);
        gtk_table_set_row_spacings (GTK_TABLE (table_keyboard), 0);
        gtk_table_set_col_spacings (GTK_TABLE (table_keyboard), 12);

        entry_move[KEY_UP] = gtk_entry_new ();
        gtk_widget_show (entry_move[KEY_UP]);
        gtk_table_attach (GTK_TABLE (table_keyboard), entry_move[KEY_UP], 1, 2, 0, 1,
                          (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);

        entry_move[KEY_DN] = gtk_entry_new ();
        gtk_widget_show (entry_move[KEY_DN]);
        gtk_table_attach (GTK_TABLE (table_keyboard), entry_move[KEY_DN], 1, 2, 1, 2,
                          (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);

        entry_move[KEY_LT] = gtk_entry_new ();
        gtk_widget_show (entry_move[KEY_LT]);
        gtk_table_attach (GTK_TABLE (table_keyboard), entry_move[KEY_LT], 1, 2, 2, 3,
                          (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);

        entry_move[KEY_RT] = gtk_entry_new ();
        gtk_widget_show (entry_move[KEY_RT]);
        gtk_table_attach (GTK_TABLE (table_keyboard), entry_move[KEY_RT], 1, 2, 3, 4,
                          (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);

        button_move[KEY_UP] = gtk_button_new_with_mnemonic (_("Move _Up"));
        gtk_widget_show (button_move[KEY_UP]);
        gtk_table_attach (GTK_TABLE (table_keyboard), button_move[KEY_UP], 0, 1, 0, 1,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);

        button_move[KEY_DN] = gtk_button_new_with_mnemonic (_("Move _Down"));
        gtk_widget_show (button_move[KEY_DN]);
        gtk_table_attach (GTK_TABLE (table_keyboard), button_move[KEY_DN], 0, 1, 1, 2,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);

        button_move[KEY_LT] = gtk_button_new_with_mnemonic (_("Move _Left"));
        gtk_widget_show (button_move[KEY_LT]);
        gtk_table_attach (GTK_TABLE (table_keyboard), button_move[KEY_LT], 0, 1, 2, 3,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);

        button_move[KEY_RT] = gtk_button_new_with_mnemonic (_("Move _Right"));
        gtk_widget_show (button_move[KEY_RT]);
        gtk_table_attach (GTK_TABLE (table_keyboard), button_move[KEY_RT], 0, 1, 3, 4,
                          (GtkAttachOptions) (GTK_FILL),
                          (GtkAttachOptions) (0), 0, 0);




        label_keyboard = gtk_label_new_with_mnemonic (_("_Keyboard"));
        gtk_widget_show (label_keyboard);
        gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1), label_keyboard);
        gtk_label_set_justify (GTK_LABEL (label_keyboard), GTK_JUSTIFY_LEFT);

        g_signal_connect (GTK_OBJECT (button_browse), "clicked",
                          GTK_SIGNAL_FUNC (on_button_browse_clicked),
                          NULL);
#if 0
        g_signal_connect (GTK_OBJECT (checkbutton_random), "toggled",
                            GTK_SIGNAL_FUNC (on_checkbutton_random_toggled),
                            NULL);
#endif
        fill_image_menu ();
        gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu_decor), overlay);
        gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu_size), puzzle_get_size (puzzle) - 3);
        gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu_image), img_id);

        size_menu_set_sensitive (img_file[img_id].size == 0);
        key_entries_set_sensitive (FALSE);

        for (i = 0; i < 4; i++) {
                gtk_entry_set_text (GTK_ENTRY(entry_move[i]), gdk_keyval_name (key[i]));
                gtk_editable_set_editable (GTK_EDITABLE(entry_move[i]), FALSE);
                g_signal_connect (GTK_OBJECT(button_move[i]),
                                  "clicked", GTK_SIGNAL_FUNC(on_button_move_clicked), (gpointer)i);
                g_signal_connect (GTK_OBJECT(entry_move[i]),
                                  "key_press_event", GTK_SIGNAL_FUNC(on_key_selected), NULL);
        }

        gtk_widget_show (dialog_prefs);
}

